@@ -2,6 +2,43 @@ from AoE2ScenarioParser.datasets.support.info_dataset_base import InfoDatasetBas
 
 
 class BuildingInfo(InfoDatasetBase):
+    """
+    **Description**
+
+    This is an enum class which provides information about most of the buildings in the game. Information about the
+    following properties of a building is found in this class:
+     - Unit ID
+     - Icon ID
+     - Dead Unit ID
+     - HotKey ID
+     - If the building is a gaia only unit (eg. ruins)
+
+    **Inherited Methods from class InfoDatasetBase**
+
+    >>> InfoDatasetBase.from_id()
+    >>> InfoDatasetBase.from_dead_id()
+    >>> InfoDatasetBase.from_icon_id()
+    >>> InfoDatasetBase.from_hotkey_id()
+    >>> InfoDatasetBase.gaia_only()
+    >>> InfoDatasetBase.non_gaia()
+
+    **Examples**
+
+    >>> BuildingInfo.RUINS.ID
+    >>> 345
+
+    >>> BuildingInfo.ARCHERY_RANGE.ICON_ID
+    >>> 0
+
+    >>> BuildingInfo.BARRACKS.DEAD_ID
+    >>> 1402
+
+    >>> BuildingInfo.BLACKSMITH.HOTKEY_ID
+    >>> 16131
+
+    >>> BuildingInfo.RUINS.IS_GAIA_ONLY
+    >>> True
+    """
     RUINS = 345, -1, -1, 16393, True
     AACHEN_CATHEDRAL = 1622, 37, 1517, 16182, False
     AMPHITHEATRE = 251, 58, 1514, 16572, False
@@ -151,3 +188,14 @@ class BuildingInfo(InfoDatasetBase):
     TRADE_WORKSHOP_BR = 1647, 17, 1429, 16174, False
     TRADE_WORKSHOP_TG = 179, 17, 1429, 16174, False
     TOWN_CENTER_PACKED = 444, 264, 1274, 16311, False
+    FOLWARK = 1734, 86, 1528, 16157, False
+    FORTIFIED_GATE_SOUTHWEST_TO_NORTHEAST = 63, 36, 1504, 16185, False
+    FORTIFIED_GATE_NORTHWEST_TO_SOUTHEAST = 85, 36, 1505, 16185, False
+    FORTIFIED_GATE_WEST_TO_EAST = 660, 36, 1506, 16185, False
+    FORTIFIED_GATE_NORTH_TO_SOUTH = 668, 36, 1507, 16185, False
+    PAGAN_SHRINE = 1712, 85, 1526, 16159, False
+    GATE = 487, 36, 1406, 16185, False
+    PALISADE_GATE = 792, 44, 1406, 16186, False
+    FORTIFIED_GATE = 488, 36, 1406, 16185, False
+    CARAVANSERAI = 1754, 87, 1529, 16351, False
+    MINARET_OF_JAM = 1773, 37, 1530, 16182, False
