@@ -1,9 +1,16 @@
 from enum import IntEnum
 
-from bidict import bidict
-
 
 class ConditionId(IntEnum):
+    """
+    This enum class provides the integer values used to reference the conditions in the game. Used in every condition
+    to indicate which type of condition it is
+
+    **Examples**
+
+    >>> ConditionId.BRING_OBJECT_TO_AREA
+    >>> 1
+    """
     NONE = 0
     """
     Attributes for the **none**  condition are:\n
@@ -34,6 +41,7 @@ class ConditionId(IntEnum):
     - source_player
     - object_group
     - object_type
+    - include_changeable_weapon_objects
     """
     OWN_FEWER_OBJECTS = 4
     """
@@ -47,6 +55,7 @@ class ConditionId(IntEnum):
     - area_y2
     - object_group
     - object_type
+    - include_changeable_weapon_objects
     """
     OBJECTS_IN_AREA = 5
     """
@@ -60,7 +69,9 @@ class ConditionId(IntEnum):
     - area_y2
     - object_group
     - object_type
+    - object_state
     - inverted
+    - include_changeable_weapon_objects
     """
     DESTROY_OBJECT = 6
     """
@@ -198,19 +209,9 @@ class ConditionId(IntEnum):
     - xs_function
     
     **Version notice**: \n
-    This condition is supported since: 1.40
+    This condition was added in: 1.40
     """
-    OBJECT_VISIBLE_MULTIPLAYER = 26
-    """
-    Attributes for the **object_visible_multiplayer** condition are:\n
-    - unit_object
-    - source_player
-    - inverted
-    
-    **Version notice**: \n
-    This condition is supported since: 1.40
-    """
-    OBJECT_SELECTED_MULTIPLAYER = 27
+    OBJECT_SELECTED_MULTIPLAYER = 26
     """
     Attributes for the **object_selected_multiplayer** condition are:\n
     - unit_object
@@ -218,7 +219,17 @@ class ConditionId(IntEnum):
     - inverted
     
     **Version notice**: \n
-    This condition is supported since: 1.40
+    This condition was added in: 1.40
+    """
+    OBJECT_VISIBLE_MULTIPLAYER = 27
+    """
+    Attributes for the **object_visible_multiplayer** condition are:\n
+    - unit_object
+    - source_player
+    - inverted
+    
+    **Version notice**: \n
+    This condition was added in: 1.40
     """
     OBJECT_HAS_ACTION = 28
     """
@@ -229,14 +240,14 @@ class ConditionId(IntEnum):
     - unit_ai_action
     
     **Version notice**: \n
-    This condition is supported since: 1.40
+    This condition was added in: 1.40
     """
     OR = 29
     """
     The **OR** condition does not have any attributes
     
     **Version notice**: \n
-    This condition is supported since: 1.40
+    This condition was added in: 1.40
     """
     AI_SIGNAL_MULTIPLAYER = 30
     """
@@ -245,7 +256,44 @@ class ConditionId(IntEnum):
     - inverted
     
     **Version notice**: \n
-    This condition is supported since: 1.40
+    This condition was added in: 1.40
+    """
+    BUILDING_IS_TRADING = 54
+    """
+    Attributes for the **building_is_trading** condition are:\n
+    - unit_object
+    - inverted
+    
+    **Version notice**: \n
+    This condition was added in: 1.46
+    """
+    DISPLAY_TIMER_TRIGGERED = 55
+    """
+    Attributes for the **display_timer_triggered** condition are:\n
+    - timer_id
+    - inverted
+    
+    **Version notice**: \n
+    This condition was added in: 1.46
+    """
+    VICTORY_TIMER = 56
+    """
+    Attributes for the **victory_timer** condition are:\n
+    - quantity
+    - source_player
+    - inverted
+    - comparison
+    - victory_timer_type
+    
+    **Version notice**: \n
+    This condition was added in: 1.46
+    """
+    AND = 57
+    """
+    The **AND** condition does not have any attributes
+    
+    **Version notice**: \n
+    This condition was added in: 1.46
     """
 
 
@@ -271,10 +319,15 @@ empty_attributes = {
     "comparison": -1,
     "target_player": -1,
     "unit_ai_action": -1,
+    "object_state": -1,
+    "timer_id": -1,
+    "victory_timer_type": -1,
+    "include_changeable_weapon_objects": -1,
     "xs_function": ""
 }
 
 # Set using the version json files
-condition_names = bidict()
+condition_names = {}
 default_attributes = {}
 attributes = {}
+attribute_presentation = {}
