@@ -2,8 +2,45 @@ from AoE2ScenarioParser.datasets.support.info_dataset_base import InfoDatasetBas
 
 
 class HeroInfo(InfoDatasetBase):
+    """
+    **Description**
+
+    This enum class provides information about most of the heroes in the game. Information about the following
+    properties of a hero is found in this class:
+     - Unit ID
+     - Icon ID
+     - Dead Unit ID
+     - HotKey ID
+     - If the hero is a gaia only unit
+
+    **Inherited Methods from class InfoDatasetBase**
+
+    >>> InfoDatasetBase.from_id()
+    >>> InfoDatasetBase.from_dead_id()
+    >>> InfoDatasetBase.from_icon_id()
+    >>> InfoDatasetBase.from_hotkey_id()
+    >>> InfoDatasetBase.gaia_only()
+    >>> InfoDatasetBase.non_gaia()
+
+    **Examples**
+
+    >>> HeroInfo.WILLIAM_WALLACE.ID
+    >>> 432
+
+    >>> HeroInfo.WILLIAM_WALLACE.ICON_ID
+    >>> 363
+
+    >>> HeroInfo.WILLIAM_WALLACE.DEAD_ID
+    >>> 433
+
+    >>> HeroInfo.WILLIAM_WALLACE.HOTKEY_ID
+    >>> 16299
+
+    >>> HeroInfo.WILLIAM_WALLACE.IS_GAIA_ONLY
+    >>> False
+    """
     KHOSRAU = 1297, 172, 874, 16311, False
-    LIEF_ERIKSON = 106, 118, -1, 16457, False
+    LEIF_ERIKSON = 106, 118, -1, 16457, False
     RICHARD_THE_LIONHEART = 160, 92, 570, 16633, False
     THE_BLACK_PRINCE = 161, 343, 139, 16622, False
     FRIAR_TUCK = 163, 311, 134, 16626, False
@@ -34,7 +71,7 @@ class HeroInfo(InfoDatasetBase):
     HENRY_THE_LION = 418, 328, 111, 16531, False
     CHARLES_MARTEL = 424, 129, 157, 16291, False
     FRANCISCO_DE_ORELLANA = 425, 153, 772, 16689, False
-    HAROLD_HARDRADA = 426, 118, 693, 16293, False
+    HARALD_HARDRADA = 426, 118, 693, 16293, False
     GONZALO_PIZARRO = 427, 167, 772, 16689, False
     HROLF_THE_GANGER = 428, 302, 695, 16295, False
     FREDERICK_BARBAROSSA = 429, 154, 181, 16462, False
@@ -47,7 +84,7 @@ class HeroInfo(InfoDatasetBase):
     FRANKISH_PALADIN = 632, 62, 633, 16529, False
     SIEUR_DE_METZ = 634, 59, 111, 16531, False
     SIEUR_BERTRAND = 636, 61, 111, 16533, False
-    DUKE_D_ALENASONN = 638, 65, 633, 16535, False
+    DUKE_D_ALENCON = 638, 65, 633, 16535, False
     LA_HIRE = 640, 51, 568, 16537, False
     LORD_DE_GRAVILLE = 642, 66, 496, 16539, False
     JEAN_DE_LORRAIN = 644, 67, 16, 16541, False
@@ -92,7 +129,7 @@ class HeroInfo(InfoDatasetBase):
     TRISTAN = 1038, 322, 870, 16779, False
     PRINCESS_YODIT = 1039, 216, 1328, 16781, False
     HENRY_II = 1040, 325, 500, 16780, False
-    YEKUNA_AMLAK = 1064, 208, 1017, 16799, False
+    YEKUNO_AMLAK = 1064, 208, 1017, 16799, False
     YODIT = 1066, 216, 1625, 16220, False
     ITZCOATL = 1067, 317, 1221, 16206, False
     MUSTAFA_PASHA = 1068, 318, 107, 16207, False
@@ -140,7 +177,7 @@ class HeroInfo(InfoDatasetBase):
     CHAND_BARDAI = 1295, 171, 1329, 16742, False
     SALADIN = 1296, 175, 44, 16453, False
     JARL = 1298, 188, 1628, 16416, False
-    SAVAR = 1299, 181, 1401, 16451, False
+    SOGDIAN_CATAPHRACT = 1299, 181, 1401, 16451, False
     OSMAN = 1303, 180, 1632, 16412, False
     MOUNTED_SAMURAI = 1568, 103, 1569, 16661, False
     SOSSO_GUARD = 1574, 350, 1575, 16101, False
@@ -158,3 +195,22 @@ class HeroInfo(InfoDatasetBase):
     WARWOLF_TREBUCHET = 1690, 94, 194, 16097, False
     WARWOLF_TREBUCHET_PACKED = 1691, 313, 735, 16381, False
     JACQUELINE_OF_HAINAUT = 1692, 367, 1328, 16307, False
+    STOERTEBEKER = 114, 88, -1, 16703, False
+    JAN_ZIZKA = 1713, 373, 1714, 16724, False
+    JADWIGA = 1715, 374, 1716, 16691, False
+    JOGAILA = 1718, 375, 1719, 16724, False
+    KESTUTIS = 1721, 376, 1722, 16724, False
+    ALGIRDAS = 1725, 378, 1726, 16724, False
+    ULRICH_VON_JUNGINGEN = 1727, 379, 1728, 16562, False
+    EMPEROR_SIGISMUND = 1729, 380, 633, 16562, False
+    DMITRY_OF_MOSCOW = 1730, 381, 139, 16639, False
+    MIKHAIL_OF_TVER = 1731, 382, 877, 16639, False
+    YOUNG_JADWIGA = 1732, 383, 431, 16297, False
+    MIHIRA_BHOJA = 1762, 396, 870, 16779, False
+    AMOGHAVARSHA = 1763, 397, 1736, 16779, False
+    RAJENDRA_CHOLA = 1764, 398, 1765, 16633, False
+    GENERAL_ARAIYAN = 1766, 399, 1767, 16218, False
+    YOUNG_BABUR = 1768, 400, 495, 16218, False
+    QUTLUGH = 1769, 401, 1770, 16637, False
+    IBRAHIM_LODI = 1771, 402, 1277, 16458, False
+    SHAYBANI_KHAN = 1772, 403, 1289, 16458, False
