@@ -1,10 +1,13 @@
 from __future__ import annotations
 
 from enum import IntEnum
+from typing import Union
 
 from AoE2ScenarioParser.datasets import conditions
-from AoE2ScenarioParser.helper.helper import raise_if_not_int_subclass
+from AoE2ScenarioParser.helper.helper import raise_if_not_int_subclass, validate_coords
+from AoE2ScenarioParser.helper.string_manipulations import add_tabs
 from AoE2ScenarioParser.objects.aoe2_object import AoE2Object
+from AoE2ScenarioParser.objects.support.attr_presentation import transform_condition_attr_value
 from AoE2ScenarioParser.sections.retrievers.retriever_object_link import RetrieverObjectLink
 from AoE2ScenarioParser.sections.retrievers.support import Support
 
@@ -45,12 +48,18 @@ class Condition(AoE2Object):
                             "trigger_data[__index__].condition_data[__index__].comparison"),
         RetrieverObjectLink("target_player", "Triggers",
                             "trigger_data[__index__].condition_data[__index__].target_player"),
-        RetrieverObjectLink("object_state", "Triggers",
-                            "trigger_data[__index__].condition_data[__index__].object_state", Support(since=1.42)),
-        RetrieverObjectLink("xs_function", "Triggers",
-                            "trigger_data[__index__].condition_data[__index__].xs_function", Support(since=1.40)),
         RetrieverObjectLink("unit_ai_action", "Triggers",
                             "trigger_data[__index__].condition_data[__index__].unit_ai_action", Support(since=1.40)),
+        RetrieverObjectLink("object_state", "Triggers",
+                            "trigger_data[__index__].condition_data[__index__].object_state", Support(since=1.42)),
+        RetrieverObjectLink("timer_id", "Triggers",
+                            "trigger_data[__index__].condition_data[__index__].timer_id", Support(since=1.46)),
+        RetrieverObjectLink("victory_timer_type", "Triggers",
+                            "trigger_data[__index__].condition_data[__index__].victory_timer_type", Support(since=1.46)),
+        RetrieverObjectLink("include_changeable_weapon_objects", "Triggers",
+                            "trigger_data[__index__].condition_data[__index__].include_changeable_weapon_objects", Support(since=1.46)),
+        RetrieverObjectLink("xs_function", "Triggers",
+                            "trigger_data[__index__].condition_data[__index__].xs_function", Support(since=1.40)),
     ]
 
     def __init__(self,
@@ -60,8 +69,8 @@ class Condition(AoE2Object):
                  unit_object: int = None,
                  next_object: int = None,
                  object_list: int = None,
-                 source_player: IntEnum = None,
-                 technology: IntEnum = None,
+                 source_player: Union[int, IntEnum] = None,
+                 technology: Union[int, IntEnum] = None,
                  timer: int = None,
                  area_x1: int = None,
                  area_y1: int = None,
@@ -73,12 +82,17 @@ class Condition(AoE2Object):
                  inverted: int = None,
                  variable: int = None,
                  comparison: int = None,
-                 target_player: IntEnum = None,
+                 target_player: Union[int, IntEnum] = None,
                  unit_ai_action: int = None,
                  object_state: int = None,
+                 timer_id: int = None,
+                 victory_timer_type: int = None,
+                 include_changeable_weapon_objects: int = None,
                  xs_function: str = None,
+                 **kwargs
                  ):
         raise_if_not_int_subclass([object_list, technology])
+        area_x1, area_y1, area_x2, area_y2 = validate_coords(area_x1, area_y1, area_x2, area_y2)
 
         self.condition_type: int = condition_type
         self.quantity: int = quantity
@@ -102,11 +116,14 @@ class Condition(AoE2Object):
         self.target_player: int = target_player
         self.unit_ai_action: int = unit_ai_action
         self.object_state: int = object_state
+        self.timer_id: int = timer_id
+        self.victory_timer_type: int = victory_timer_type
+        self.include_changeable_weapon_objects: int = include_changeable_weapon_objects
         self.xs_function: str = xs_function
 
-        super().__init__()
+        super().__init__(**kwargs)
 
-    def get_content_as_string(self) -> str:
+    def get_content_as_string(self, include_effect_definition=False) -> str:
         if self.condition_type not in conditions.attributes:
             attributes_list = conditions.empty_attributes
         else:
@@ -114,12 +131,20 @@ class Condition(AoE2Object):
 
         return_string = ""
         for attribute in attributes_list:
-            attr = getattr(self, attribute)
-            if attribute == "condition_type" or attr in [[], [-1], [''], "", " ", -1]:
+            val = getattr(self, attribute)
+            if attribute == "condition_type" or val in [[], [-1], [''], "", " ", -1]:
                 continue
-            return_string += "\t\t\t\t" + attribute + ": " + str(attr) + "\n"
+
+            value_string = transform_condition_attr_value(self.condition_type, attribute, val, self._host_uuid)
+            return_string += f"{attribute}: {value_string}\n"
 
         if return_string == "":
-            return "\t\t\t\t<< No Attributes >>\n"
+            return "<< No Attributes >>\n"
+
+        if include_effect_definition:
+            return f"{conditions.condition_names[self.condition_type]}:\n{add_tabs(return_string, 1)}"
 
         return return_string
+
+    def __str__(self):
+        return f"[Condition] {self.get_content_as_string(include_effect_definition=True)}"
