@@ -1,19 +1,27 @@
 from __future__ import annotations
 
-from typing import List, Tuple
+from typing import List, Tuple, Union
 
 from AoE2ScenarioParser.datasets import effects
 from AoE2ScenarioParser.datasets.effects import EffectId
+from AoE2ScenarioParser.datasets.players import PlayerColorId, PlayerId
 from AoE2ScenarioParser.datasets.trigger_lists import ObjectAttribute
-from AoE2ScenarioParser.helper.bytes_conversions import int_to_bytes, bytes_to_int
-from AoE2ScenarioParser.helper.helper import raise_if_not_int_subclass
-from AoE2ScenarioParser.helper.list_functions import listify
+from AoE2ScenarioParser.helper.helper import raise_if_not_int_subclass, value_is_valid, validate_coords
 from AoE2ScenarioParser.helper.printers import warn
+from AoE2ScenarioParser.helper.string_manipulations import add_tabs
 from AoE2ScenarioParser.objects.aoe2_object import AoE2Object
+from AoE2ScenarioParser.objects.support.attr_presentation import transform_effect_attr_value
+from AoE2ScenarioParser.scenarios.scenario_store import getters
 from AoE2ScenarioParser.sections.retrievers.retriever_object_link import RetrieverObjectLink
 from AoE2ScenarioParser.sections.retrievers.support import Support
 
 
+def _add_trail_if_string_attr_is_used_in_effect(obj: Effect, attr_name, val: Union[bytes, str]):
+    if attr_name in effects.attributes[obj.effect_type]:
+        return val + (b"\x00" if type(val) is bytes else "\x00")
+    return val
+
+
 class Effect(AoE2Object):
     """Object for handling an effect."""
 
@@ -24,6 +32,8 @@ class Effect(AoE2Object):
         RetrieverObjectLink("quantity", "Triggers", "trigger_data[__index__].effect_data[__index__].quantity"),
         RetrieverObjectLink("tribute_list", "Triggers", "trigger_data[__index__].effect_data[__index__].tribute_list"),
         RetrieverObjectLink("diplomacy", "Triggers", "trigger_data[__index__].effect_data[__index__].diplomacy"),
+        RetrieverObjectLink("legacy_location_object_reference", "Triggers",
+                            "trigger_data[__index__].effect_data[__index__].legacy_location_object_reference"),
         RetrieverObjectLink("object_list_unit_id", "Triggers",
                             "trigger_data[__index__].effect_data[__index__].object_list_unit_id"),
         RetrieverObjectLink("source_player", "Triggers",
@@ -81,8 +91,16 @@ class Effect(AoE2Object):
                             Support(since=1.40)),
         RetrieverObjectLink("color_mood", "Triggers", "trigger_data[__index__].effect_data[__index__].color_mood",
                             Support(since=1.42)),
-        RetrieverObjectLink("message", "Triggers", "trigger_data[__index__].effect_data[__index__].message"),
-        RetrieverObjectLink("sound_name", "Triggers", "trigger_data[__index__].effect_data[__index__].sound_name"),
+        RetrieverObjectLink("reset_timer", "Triggers", "trigger_data[__index__].effect_data[__index__].reset_timer",
+                            Support(since=1.44)),
+        RetrieverObjectLink("object_state", "Triggers", "trigger_data[__index__].effect_data[__index__].object_state",
+                            Support(since=1.44)),
+        RetrieverObjectLink("action_type", "Triggers", "trigger_data[__index__].effect_data[__index__].action_type",
+                            Support(since=1.44)),
+        RetrieverObjectLink("message", "Triggers", "trigger_data[__index__].effect_data[__index__].message",
+                            commit_callback=_add_trail_if_string_attr_is_used_in_effect),
+        RetrieverObjectLink("sound_name", "Triggers", "trigger_data[__index__].effect_data[__index__].sound_name",
+                            commit_callback=_add_trail_if_string_attr_is_used_in_effect),
         RetrieverObjectLink("selected_object_ids", "Triggers",
                             "trigger_data[__index__].effect_data[__index__].selected_object_ids"),
     ]
@@ -95,6 +113,7 @@ class Effect(AoE2Object):
                  quantity: int = None,
                  tribute_list: int = None,
                  diplomacy: int = None,
+                 legacy_location_object_reference: int = None,
                  object_list_unit_id: int = None,
                  source_player: int = None,
                  target_player: int = None,
@@ -119,7 +138,7 @@ class Effect(AoE2Object):
                  wood: int = None,
                  stone: int = None,
                  gold: int = None,
-                 item_id: int = None,
+                 item_id: int = None,  # Unused (?)
                  flash_object: int = None,
                  force_research_technology: int = None,
                  visibility_state: int = None,
@@ -135,21 +154,45 @@ class Effect(AoE2Object):
                  play_sound: int = None,
                  player_color: int = None,
                  color_mood: int = None,
+                 reset_timer: int = None,
+                 object_state: int = None,
+                 action_type: int = None,
                  message: str = None,
                  sound_name: str = None,
                  selected_object_ids: List[int] = None,
+                 **kwargs
                  ):
+        super().__init__(**kwargs)
+
         raise_if_not_int_subclass([object_list_unit_id, technology, object_list_unit_id_2])
 
+        if selected_object_ids is None:
+            selected_object_ids = []
+
         # Set flags
         self._armour_attack_flag = _set_armour_attack_flag(effect_type, object_attributes)
 
-        if self._armour_attack_flag and quantity is not None:
-            armour_attack_quantity, armour_attack_class = _quantity_to_aa(quantity)
-            quantity = None
+        # Handle armour effect attributes:
+        #   When effect is created through trigger.new_effect, aa values will be -1.
+        #   If created while reading a scenario, they both default to None.
+        if self._armour_attack_flag:
+            # If effect created through reading scenario file
+            if quantity is not None and armour_attack_class is None and armour_attack_quantity is None:
+                armour_attack_class, armour_attack_quantity = self._quantity_to_aa(quantity)
+                quantity = None
+            # If effect created through new_effect with aa values defined
+            elif value_is_valid(armour_attack_class) or value_is_valid(armour_attack_quantity):
+                quantity = None
+            # If created through new_effect with quantity defined instead of the aa values. Handled by quantity property
+            else:
+                pass
+        else:
+            armour_attack_class = armour_attack_quantity = None
 
-        if selected_object_ids is None:
-            selected_object_ids = []
+        area_x1, area_y1, area_x2, area_y2 = validate_coords(area_x1, area_y1, area_x2, area_y2)
+
+        if value_is_valid(legacy_location_object_reference):
+            location_object_reference = legacy_location_object_reference
 
         # Bypass the @property which causes: self._update_armour_attack_flag()
         self._effect_type: int = effect_type
@@ -183,7 +226,7 @@ class Effect(AoE2Object):
         self.wood: int = wood
         self.stone: int = stone
         self.gold: int = gold
-        self.item_id: int = item_id
+        # self.item_id: int = item_id  # Unused (?)
         self.flash_object: int = flash_object
         self.force_research_technology: int = force_research_technology
         self.visibility_state: int = visibility_state
@@ -199,11 +242,35 @@ class Effect(AoE2Object):
         self.play_sound: int = play_sound
         self.player_color: int = player_color
         self.color_mood: int = color_mood
+        self.reset_timer = reset_timer
+        self.object_state = object_state
+        self.action_type = action_type
         self.message: str = message
         self.sound_name: str = sound_name
         self.selected_object_ids: List[int] = selected_object_ids
 
-        super().__init__()
+    @property
+    def legacy_location_object_reference(self) -> int:
+        return -1
+
+    @property
+    def player_color(self):
+        return self._player_color
+
+    @player_color.setter
+    def player_color(self, value):
+        if type(value) in [PlayerColorId, PlayerId]:
+            value -= 1
+        self._player_color = value
+
+    @property
+    def item_id(self):
+        return self.object_list_unit_id
+
+    @item_id.setter
+    def item_id(self, value):
+        raise ValueError("The `item_id` attribute isn't used in scenario's. Please use `object_list_unit_id`.\n"
+                         "If you believe this is an error, please create a report at GitHub.")
 
     @property
     def effect_type(self):
@@ -230,7 +297,8 @@ class Effect(AoE2Object):
     @armour_attack_quantity.setter
     def armour_attack_quantity(self, value):
         if value is not None and value != [] and not self._armour_attack_flag:
-            warn("Setting 'armour_attack_quantity' when the effect doesn't use armour/attack attributes might result in unintended behaviour.")
+            warn("Setting 'effect.armour_attack_quantity' when the effect doesn't use armour/attack attributes "
+                 "might result in unintended behaviour.")
         self._armour_attack_quantity = value
 
     @property
@@ -240,21 +308,24 @@ class Effect(AoE2Object):
     @armour_attack_class.setter
     def armour_attack_class(self, value):
         if value is not None and value != [] and not self._armour_attack_flag:
-            warn("Setting 'armour_attack_class' when the effect doesn't use armour/attack attributes might result in unintended behaviour.")
+            warn("Setting 'effect.armour_attack_class' when the effect doesn't use armour/attack attributes "
+                 "might result in unintended behaviour.")
         self._armour_attack_class = value
 
     @property
     def quantity(self):
         if self._armour_attack_flag:
-            return _aa_to_quantity(self.armour_attack_quantity, self.armour_attack_class)
+            return self._aa_to_quantity(self.armour_attack_quantity, self.armour_attack_class)
         return self._quantity
 
     @quantity.setter
     def quantity(self, value):
         # Quantity by default, when unused is []
         if value is not None and value != [] and self._armour_attack_flag:
-            warn("Setting 'effect.quantity' directly in an effect that uses armour/attack attributes might result in unintended behaviour.\n"
-                 "Please use the 'effect.armour_attack_quantity' and 'effect.armour_attack_class' attributes instead.")
+            warn("Setting 'effect.quantity' directly in an effect that uses armour/attack attributes "
+                 "might result in unintended behaviour.\nPlease use the 'effect.armour_attack_quantity' "
+                 "and 'effect.armour_attack_class' attributes instead.")
+            self.armour_attack_class, self.armour_attack_quantity = self._quantity_to_aa(value)
         self._quantity = value
 
     @property
@@ -263,10 +334,11 @@ class Effect(AoE2Object):
 
     @selected_object_ids.setter
     def selected_object_ids(self, val: List[int]):
-        val = listify(val)
+        if type(val) is int:
+            val = [val]
         self._selected_object_ids = val
 
-    def get_content_as_string(self) -> str:
+    def get_content_as_string(self, include_effect_definition=False) -> str:
         if self.effect_type not in effects.attributes:  # Unknown effect
             attributes_list = effects.empty_attributes
         else:
@@ -274,61 +346,90 @@ class Effect(AoE2Object):
 
         return_string = ""
         for attribute in attributes_list:
-            attribute_value = getattr(self, attribute)
-            if attribute == "effect_type" or attribute_value in [[], [-1], "", " ", -1]:
+            val = getattr(self, attribute)
+            if attribute == "effect_type" or val in [[], [-1], "", " ", -1]:
                 continue
             # Ignore the quantity value in the print statement when flag is True.
             if self._armour_attack_flag and attribute == "quantity":
                 continue
-            if attribute in ["armour_attack_quantity", "armour_attack_class"] and not self._armour_attack_flag:
+            if not self._armour_attack_flag and attribute in ["armour_attack_quantity", "armour_attack_class"]:
                 continue
-            return_string += "\t\t\t\t" + attribute + ": " + str(attribute_value) + "\n"
+
+            value_string = transform_effect_attr_value(self.effect_type, attribute, val, self._host_uuid)
+            return_string += f"{attribute}: {value_string}\n"
 
         if return_string == "":
-            return "\t\t\t\t<< No Attributes >>\n"
+            return "<< No Attributes >>\n"
 
+        if include_effect_definition:
+            return f"{effects.effect_names[self.effect_type]}:\n{add_tabs(return_string, 1)}"
         return return_string
 
     def _update_armour_attack_flag(self):
         self._armour_attack_flag = _set_armour_attack_flag(self.effect_type, self.object_attributes)
 
+    def _quantity_to_aa(self, quantity: int) -> Tuple[int, int]:
+        """
+        A function to convert the initial quantity value to the quantity and armor/attack values.
+        Unfortunately this problem has to be solved in the object due to how specific this was implemented in DE.
+
+        Args:
+            quantity (int): the initial quantity value
+
+        Returns:
+            The one byte armor/attack class as int and one byte armor/attack quantity as int
+
+        ----
+
+        **Trigger Version 2.4**::
+
+            Quantity value: (3, 5)
+            00000000 00000000 00000011 000000101
+                                aaq      aac
+
+        Final 2/4 bytes are aaq (1 byte), and aac (1 byte). First 2 are unused. Max value of both is 255.
+
+        **Trigger Version 2.5**::
+
+            Quantity value: (3, 5)
+            00000000 00000011 00000000 000000101
+              aaq      aaq      aac      aac
+
+        The 4/4 bytes are aaq (2 bytes), and aac (2 bytes). All are used. Max value of both is 65535.
+
+        ----
+        """
+        trigger_version = getters.get_trigger_version(self._host_uuid)
+        if trigger_version == 2.4:
+            return quantity >> 8, quantity & 255
+        elif trigger_version >= 2.5:
+            return quantity >> 16, quantity & 65535
+
+    def _aa_to_quantity(self, aa_quantity: int, aa_class: int) -> int:
+        """
+        A function to convert the quantity and armor/attack field to a quantity value.
+        Unfortunately this problem has to be solved in the object due to how specific this was implemented in DE.
+
+        Args:
+            aa_quantity (int): the armor quantity value
+            aa_class (int): the armor/attack value
+
+        Returns:
+            The one byte quantity and one byte armor/attack value
+        """
+        trigger_version = getters.get_trigger_version(self._host_uuid)
+        if trigger_version == 2.4:
+            # Would use `aa_class << 8` - but apparently multiplication is faster
+            return aa_class * 256 + aa_quantity
+        elif trigger_version >= 2.5:
+            return aa_class * 65536 + aa_quantity
+
+    def __str__(self):
+        return f"[Effect] {self.get_content_as_string(include_effect_definition=True)}"
+
 
 def _set_armour_attack_flag(effect_type, object_attributes) -> bool:
     return (effect_type in [EffectId.CHANGE_OBJECT_ATTACK, EffectId.CHANGE_OBJECT_ARMOR]) or \
-    (effect_type == EffectId.MODIFY_ATTRIBUTE and object_attributes in [
-        ObjectAttribute.ATTACK, ObjectAttribute.ARMOR
-    ])
-
-
-def _quantity_to_aa(quantity: int) -> Tuple[int, int]:
-    """
-    A function to convert the initial quantity value to the quantity and armor/attack values.
-    Unfortunately this problem has to be solved in the object due to how specific this was implemented in DE.
-
-    Quantity value: (3, 5)
-    00000000 00000000 00000011 000000101
-                        aaq      aac
-
-    Args:
-        quantity (int): the initial quantity value
-
-    Returns:
-        The one byte armor/attack quantity as int and one byte armor/attack class as int
-    """
-    return quantity >> 8, quantity & 255
-
-def _aa_to_quantity(aa_quantity: int, aa_class: int) -> int:
-    """
-    A function to convert the quantity and armor/attack field to a quantity value.
-    Unfortunately this problem has to be solved in the object due to how specific this was implemented in DE.
-
-
-    Args:
-        aa_quantity (int): the armor quantity value
-        aa_class (int): the armor/attack value
-
-    Returns:
-        The one byte quantity and one byte armor/attack value
-    """
-    # Would use `aa_quantity << 8` - but apparently multiplication is faster
-    return aa_quantity * 256 + aa_class
+           (effect_type == EffectId.MODIFY_ATTRIBUTE and object_attributes in [
+               ObjectAttribute.ATTACK, ObjectAttribute.ARMOR
+           ])
