@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import math
+from typing import Union
 
 from AoE2ScenarioParser.datasets.players import PlayerId
 from AoE2ScenarioParser.helper import helper
@@ -8,6 +9,7 @@ from AoE2ScenarioParser.helper.helper import raise_if_not_int_subclass
 from AoE2ScenarioParser.helper.pretty_format import pretty_format_name
 from AoE2ScenarioParser.objects.aoe2_object import AoE2Object
 from AoE2ScenarioParser.objects.support.tile import Tile
+from AoE2ScenarioParser.scenarios.scenario_store import actions
 from AoE2ScenarioParser.sections.retrievers.retriever_object_link import RetrieverObjectLink
 
 
@@ -27,7 +29,7 @@ class Unit(AoE2Object):
     ]
 
     def __init__(self,
-                 player: PlayerId,
+                 player: Union[int, PlayerId],
                  x: float,
                  y: float,
                  z: float,
@@ -36,18 +38,14 @@ class Unit(AoE2Object):
                  status: int,
                  rotation: float,
                  initial_animation_frame: int,
-                 garrisoned_in_id: int
+                 garrisoned_in_id: int,
+                 **kwargs
                  ):
         raise_if_not_int_subclass([unit_const])
 
+        super().__init__(**kwargs)
+
         self._player: PlayerId = PlayerId(player)
-        """
-        PLEASE NOTE: This is an internal (read-only) value for ease of access. It accurately represent the actual 
-        player controlling the unit but is not directly connected to it. Changing this value will have no impact to your
-        scenario.
-        To change which player controls this unit, use:
-            unit_manager.change_ownership(Unit, to_player)
-        """
         self.x: float = x
         self.y: float = y
         self.z: float = z
@@ -58,19 +56,15 @@ class Unit(AoE2Object):
         self.initial_animation_frame: int = initial_animation_frame
         self.garrisoned_in_id: int = garrisoned_in_id
 
-        super().__init__()
-
     @property
     def player(self) -> PlayerId:
-        """
-        PLEASE NOTE: This is an internal (read-only) value for ease of access. It DOES accurately represent the actual
-        player controlling the unit BUT IT IS NOT directly connected to it. Changing this value will have no impact to
-        your scenario.
-        To change which player controls this unit, use:
-            unit_manager.change_ownership(Unit, to_player)
-        """
         return self._player
 
+    @player.setter
+    def player(self, player: Union[int, PlayerId]):
+        actions.unit_change_ownership(self._host_uuid, player, self)
+        self._player = player
+
     @property
     def tile(self) -> Tile:
         return Tile(math.floor(self.x), math.floor(self.y))
