@@ -7,12 +7,12 @@ from AoE2ScenarioParser.objects.data_objects.condition import Condition
 
 class NewConditionSupport:
     def __init__(self, trigger_ref):
-        self.trigger_ref = trigger_ref
+        self._trigger_ref = trigger_ref
 
     def none(
             self,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.NONE,
         )
 
@@ -25,7 +25,7 @@ class NewConditionSupport:
             area_y2: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.BRING_OBJECT_TO_AREA,
             unit_object=unit_object,
             area_x1=area_x1,
@@ -41,7 +41,7 @@ class NewConditionSupport:
             next_object: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.BRING_OBJECT_TO_OBJECT,
             unit_object=unit_object,
             next_object=next_object,
@@ -55,14 +55,16 @@ class NewConditionSupport:
             source_player: Union[int, None] = None,
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
+            include_changeable_weapon_objects: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.OWN_OBJECTS,
             quantity=quantity,
             object_list=object_list,
             source_player=source_player,
             object_group=object_group,
             object_type=object_type,
+            include_changeable_weapon_objects=include_changeable_weapon_objects,
         )
 
     def own_fewer_objects(
@@ -76,8 +78,9 @@ class NewConditionSupport:
             area_y2: Union[int, None] = None,
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
+            include_changeable_weapon_objects: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.OWN_FEWER_OBJECTS,
             quantity=quantity,
             object_list=object_list,
@@ -88,6 +91,7 @@ class NewConditionSupport:
             area_y2=area_y2,
             object_group=object_group,
             object_type=object_type,
+            include_changeable_weapon_objects=include_changeable_weapon_objects,
         )
 
     def objects_in_area(
@@ -103,8 +107,9 @@ class NewConditionSupport:
             object_type: Union[int, None] = None,
             inverted: Union[int, None] = None,
             object_state: Union[int, None] = None,
+            include_changeable_weapon_objects: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.OBJECTS_IN_AREA,
             quantity=quantity,
             object_list=object_list,
@@ -117,6 +122,7 @@ class NewConditionSupport:
             object_type=object_type,
             inverted=inverted,
             object_state=object_state,
+            include_changeable_weapon_objects=include_changeable_weapon_objects,
         )
 
     def destroy_object(
@@ -124,7 +130,7 @@ class NewConditionSupport:
             unit_object: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.DESTROY_OBJECT,
             unit_object=unit_object,
             inverted=inverted,
@@ -136,7 +142,7 @@ class NewConditionSupport:
             source_player: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.CAPTURE_OBJECT,
             unit_object=unit_object,
             source_player=source_player,
@@ -150,7 +156,7 @@ class NewConditionSupport:
             source_player: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.ACCUMULATE_ATTRIBUTE,
             quantity=quantity,
             attribute=attribute,
@@ -164,7 +170,7 @@ class NewConditionSupport:
             technology: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.RESEARCH_TECHNOLOGY,
             source_player=source_player,
             technology=technology,
@@ -176,7 +182,7 @@ class NewConditionSupport:
             timer: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.TIMER,
             timer=timer,
             inverted=inverted,
@@ -187,7 +193,7 @@ class NewConditionSupport:
             unit_object: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.OBJECT_SELECTED,
             unit_object=unit_object,
             inverted=inverted,
@@ -198,7 +204,7 @@ class NewConditionSupport:
             ai_signal: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.AI_SIGNAL,
             ai_signal=ai_signal,
             inverted=inverted,
@@ -209,7 +215,7 @@ class NewConditionSupport:
             source_player: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.PLAYER_DEFEATED,
             source_player=source_player,
             inverted=inverted,
@@ -224,7 +230,7 @@ class NewConditionSupport:
             object_type: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.OBJECT_HAS_TARGET,
             unit_object=unit_object,
             next_object=next_object,
@@ -238,7 +244,7 @@ class NewConditionSupport:
             self,
             unit_object: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.OBJECT_VISIBLE,
             unit_object=unit_object,
         )
@@ -247,7 +253,7 @@ class NewConditionSupport:
             self,
             unit_object: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.OBJECT_NOT_VISIBLE,
             unit_object=unit_object,
         )
@@ -258,7 +264,7 @@ class NewConditionSupport:
             technology: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.RESEARCHING_TECH,
             source_player=source_player,
             technology=technology,
@@ -271,7 +277,7 @@ class NewConditionSupport:
             unit_object: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.UNITS_GARRISONED,
             quantity=quantity,
             unit_object=unit_object,
@@ -283,7 +289,7 @@ class NewConditionSupport:
             quantity: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.DIFFICULTY_LEVEL,
             quantity=quantity,
             inverted=inverted,
@@ -293,7 +299,7 @@ class NewConditionSupport:
             self,
             quantity: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.CHANCE,
             quantity=quantity,
         )
@@ -305,7 +311,7 @@ class NewConditionSupport:
             technology: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.TECHNOLOGY_STATE,
             quantity=quantity,
             source_player=source_player,
@@ -320,7 +326,7 @@ class NewConditionSupport:
             variable: Union[int, None] = None,
             comparison: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.VARIABLE_VALUE,
             quantity=quantity,
             inverted=inverted,
@@ -335,7 +341,7 @@ class NewConditionSupport:
             inverted: Union[int, None] = None,
             comparison: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.OBJECT_HP,
             quantity=quantity,
             unit_object=unit_object,
@@ -350,7 +356,7 @@ class NewConditionSupport:
             inverted: Union[int, None] = None,
             target_player: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.DIPLOMACY_STATE,
             quantity=quantity,
             source_player=source_player,
@@ -362,7 +368,7 @@ class NewConditionSupport:
             self,
             xs_function: Union[str, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.SCRIPT_CALL,
             xs_function=xs_function,
         )
@@ -373,7 +379,7 @@ class NewConditionSupport:
             source_player: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.OBJECT_VISIBLE_MULTIPLAYER,
             unit_object=unit_object,
             source_player=source_player,
@@ -386,7 +392,7 @@ class NewConditionSupport:
             source_player: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.OBJECT_SELECTED_MULTIPLAYER,
             unit_object=unit_object,
             source_player=source_player,
@@ -397,13 +403,19 @@ class NewConditionSupport:
             self,
             unit_object: Union[int, None] = None,
             next_object: Union[int, None] = None,
+            object_list: Union[int, None] = None,
+            object_group: Union[int, None] = None,
+            object_type: Union[int, None] = None,
             inverted: Union[int, None] = None,
             unit_ai_action: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.OBJECT_HAS_ACTION,
             unit_object=unit_object,
             next_object=next_object,
+            object_list=object_list,
+            object_group=object_group,
+            object_type=object_type,
             inverted=inverted,
             unit_ai_action=unit_ai_action,
         )
@@ -411,7 +423,7 @@ class NewConditionSupport:
     def or_(
             self,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.OR,
         )
 
@@ -420,8 +432,54 @@ class NewConditionSupport:
             ai_signal: Union[int, None] = None,
             inverted: Union[int, None] = None,
     ) -> Condition:
-        return self.trigger_ref._add_condition(
+        return self._trigger_ref._add_condition(
             ConditionId.AI_SIGNAL_MULTIPLAYER,
             ai_signal=ai_signal,
             inverted=inverted,
         )
+
+    def and_(
+            self,
+    ) -> Condition:
+        return self._trigger_ref._add_condition(
+            ConditionId.AND,
+        )
+
+    def building_is_trading(
+            self,
+            unit_object: Union[int, None] = None,
+            inverted: Union[int, None] = None,
+    ) -> Condition:
+        return self._trigger_ref._add_condition(
+            ConditionId.BUILDING_IS_TRADING,
+            unit_object=unit_object,
+            inverted=inverted,
+        )
+
+    def display_timer_triggered(
+            self,
+            timer_id: Union[int, None] = None,
+            inverted: Union[int, None] = None,
+    ) -> Condition:
+        return self._trigger_ref._add_condition(
+            ConditionId.DISPLAY_TIMER_TRIGGERED,
+            timer_id=timer_id,
+            inverted=inverted,
+        )
+
+    def victory_timer(
+            self,
+            quantity: Union[int, None] = None,
+            source_player: Union[int, None] = None,
+            inverted: Union[int, None] = None,
+            comparison: Union[int, None] = None,
+            victory_timer_type: Union[int, None] = None,
+    ) -> Condition:
+        return self._trigger_ref._add_condition(
+            ConditionId.VICTORY_TIMER,
+            quantity=quantity,
+            source_player=source_player,
+            inverted=inverted,
+            comparison=comparison,
+            victory_timer_type=victory_timer_type,
+        )
