@@ -1,5 +1,5 @@
 # This file was generated based on: versions/DE/v1.41/effects.json
-from typing import Union
+from typing import Union, List
 
 from AoE2ScenarioParser.datasets.effects import EffectId
 from AoE2ScenarioParser.objects.data_objects.effect import Effect
@@ -7,12 +7,12 @@ from AoE2ScenarioParser.objects.data_objects.effect import Effect
 
 class NewEffectSupport:
     def __init__(self, trigger_ref):
-        self.trigger_ref = trigger_ref
+        self._trigger_ref = trigger_ref
 
     def none(
             self,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.NONE,
         )
 
@@ -22,7 +22,7 @@ class NewEffectSupport:
             source_player: Union[int, None] = None,
             target_player: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_DIPLOMACY,
             diplomacy=diplomacy,
             source_player=source_player,
@@ -35,7 +35,7 @@ class NewEffectSupport:
             technology: Union[int, None] = None,
             force_research_technology: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.RESEARCH_TECHNOLOGY,
             source_player=source_player,
             technology=technology,
@@ -49,7 +49,7 @@ class NewEffectSupport:
             message: Union[str, None] = None,
             sound_name: Union[str, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.SEND_CHAT,
             source_player=source_player,
             string_id=string_id,
@@ -65,7 +65,7 @@ class NewEffectSupport:
             location_object_reference: Union[int, None] = None,
             sound_name: Union[str, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.PLAY_SOUND,
             source_player=source_player,
             location_x=location_x,
@@ -81,7 +81,7 @@ class NewEffectSupport:
             source_player: Union[int, None] = None,
             target_player: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.TRIBUTE,
             quantity=quantity,
             tribute_list=tribute_list,
@@ -91,18 +91,18 @@ class NewEffectSupport:
 
     def unlock_gate(
             self,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.UNLOCK_GATE,
             selected_object_ids=selected_object_ids,
         )
 
     def lock_gate(
             self,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.LOCK_GATE,
             selected_object_ids=selected_object_ids,
         )
@@ -111,7 +111,7 @@ class NewEffectSupport:
             self,
             trigger_id: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.ACTIVATE_TRIGGER,
             trigger_id=trigger_id,
         )
@@ -120,7 +120,7 @@ class NewEffectSupport:
             self,
             trigger_id: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.DEACTIVATE_TRIGGER,
             trigger_id=trigger_id,
         )
@@ -129,7 +129,7 @@ class NewEffectSupport:
             self,
             ai_script_goal: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.AI_SCRIPT_GOAL,
             ai_script_goal=ai_script_goal,
         )
@@ -140,16 +140,14 @@ class NewEffectSupport:
             source_player: Union[int, None] = None,
             location_x: Union[int, None] = None,
             location_y: Union[int, None] = None,
-            item_id: Union[int, None] = None,
             facet: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CREATE_OBJECT,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
             location_x=location_x,
             location_y=location_y,
-            item_id=item_id,
             facet=facet,
         )
 
@@ -166,9 +164,10 @@ class NewEffectSupport:
             area_y2: Union[int, None] = None,
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            action_type: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.TASK_OBJECT,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -181,6 +180,7 @@ class NewEffectSupport:
             area_y2=area_y2,
             object_group=object_group,
             object_type=object_type,
+            action_type=action_type,
             selected_object_ids=selected_object_ids,
         )
 
@@ -189,7 +189,7 @@ class NewEffectSupport:
             source_player: Union[int, None] = None,
             enabled: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.DECLARE_VICTORY,
             source_player=source_player,
             enabled=enabled,
@@ -205,9 +205,9 @@ class NewEffectSupport:
             area_y2: Union[int, None] = None,
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.KILL_OBJECT,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -230,9 +230,10 @@ class NewEffectSupport:
             area_y2: Union[int, None] = None,
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            object_state: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.REMOVE_OBJECT,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -242,6 +243,7 @@ class NewEffectSupport:
             area_y2=area_y2,
             object_group=object_group,
             object_type=object_type,
+            object_state=object_state,
             selected_object_ids=selected_object_ids,
         )
 
@@ -252,7 +254,7 @@ class NewEffectSupport:
             location_y: Union[int, None] = None,
             scroll: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_VIEW,
             source_player=source_player,
             location_x=location_x,
@@ -273,9 +275,9 @@ class NewEffectSupport:
             area_y2: Union[int, None] = None,
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.UNLOAD,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -303,9 +305,9 @@ class NewEffectSupport:
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
             flash_object: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_OWNERSHIP,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -326,22 +328,20 @@ class NewEffectSupport:
             source_player: Union[int, None] = None,
             location_x: Union[int, None] = None,
             location_y: Union[int, None] = None,
-            location_object_reference: Union[int, None] = None,
             area_x1: Union[int, None] = None,
             area_y1: Union[int, None] = None,
             area_x2: Union[int, None] = None,
             area_y2: Union[int, None] = None,
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.PATROL,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
             location_x=location_x,
             location_y=location_y,
-            location_object_reference=location_object_reference,
             area_x1=area_x1,
             area_y1=area_y1,
             area_x2=area_x2,
@@ -362,7 +362,7 @@ class NewEffectSupport:
             message: Union[str, None] = None,
             sound_name: Union[str, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.DISPLAY_INSTRUCTIONS,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -378,7 +378,7 @@ class NewEffectSupport:
             self,
             instruction_panel_position: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CLEAR_INSTRUCTIONS,
             instruction_panel_position=instruction_panel_position,
         )
@@ -393,9 +393,9 @@ class NewEffectSupport:
             area_y2: Union[int, None] = None,
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.FREEZE_OBJECT,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -411,7 +411,7 @@ class NewEffectSupport:
     def use_advanced_buttons(
             self,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.USE_ADVANCED_BUTTONS,
         )
 
@@ -426,9 +426,9 @@ class NewEffectSupport:
             area_y2: Union[int, None] = None,
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.DAMAGE_OBJECT,
             quantity=quantity,
             object_list_unit_id=object_list_unit_id,
@@ -449,7 +449,7 @@ class NewEffectSupport:
             location_x: Union[int, None] = None,
             location_y: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.PLACE_FOUNDATION,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -467,9 +467,9 @@ class NewEffectSupport:
             area_x2: Union[int, None] = None,
             area_y2: Union[int, None] = None,
             message: Union[str, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_OBJECT_NAME,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -494,9 +494,9 @@ class NewEffectSupport:
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
             operation: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_OBJECT_HP,
             quantity=quantity,
             object_list_unit_id=object_list_unit_id,
@@ -524,9 +524,9 @@ class NewEffectSupport:
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
             operation: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_OBJECT_ATTACK,
             armour_attack_quantity=armour_attack_quantity,
             armour_attack_class=armour_attack_class,
@@ -552,9 +552,9 @@ class NewEffectSupport:
             area_y2: Union[int, None] = None,
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.STOP_OBJECT,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -580,9 +580,9 @@ class NewEffectSupport:
             area_y2: Union[int, None] = None,
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.ATTACK_MOVE,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -611,9 +611,9 @@ class NewEffectSupport:
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
             operation: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_OBJECT_ARMOR,
             armour_attack_quantity=armour_attack_quantity,
             armour_attack_class=armour_attack_class,
@@ -641,9 +641,9 @@ class NewEffectSupport:
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
             operation: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_OBJECT_RANGE,
             quantity=quantity,
             object_list_unit_id=object_list_unit_id,
@@ -669,9 +669,9 @@ class NewEffectSupport:
             area_y2: Union[int, None] = None,
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_OBJECT_SPEED,
             quantity=quantity,
             object_list_unit_id=object_list_unit_id,
@@ -696,9 +696,9 @@ class NewEffectSupport:
             area_y2: Union[int, None] = None,
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.HEAL_OBJECT,
             quantity=quantity,
             object_list_unit_id=object_list_unit_id,
@@ -724,9 +724,9 @@ class NewEffectSupport:
             area_y2: Union[int, None] = None,
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.TELEPORT_OBJECT,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -752,9 +752,9 @@ class NewEffectSupport:
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
             attack_stance: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_OBJECT_STANCE,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -774,14 +774,16 @@ class NewEffectSupport:
             display_time: Union[int, None] = None,
             time_unit: Union[int, None] = None,
             timer: Union[int, None] = None,
+            reset_timer: Union[int, None] = None,
             message: Union[str, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.DISPLAY_TIMER,
             string_id=string_id,
             display_time=display_time,
             time_unit=time_unit,
             timer=timer,
+            reset_timer=reset_timer,
             message=message,
         )
 
@@ -790,14 +792,12 @@ class NewEffectSupport:
             object_list_unit_id: Union[int, None] = None,
             source_player: Union[int, None] = None,
             enabled: Union[int, None] = None,
-            item_id: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.ENABLE_DISABLE_OBJECT,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
             enabled=enabled,
-            item_id=item_id,
         )
 
     def enable_disable_technology(
@@ -805,14 +805,12 @@ class NewEffectSupport:
             source_player: Union[int, None] = None,
             technology: Union[int, None] = None,
             enabled: Union[int, None] = None,
-            item_id: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.ENABLE_DISABLE_TECHNOLOGY,
             source_player=source_player,
             technology=technology,
             enabled=enabled,
-            item_id=item_id,
         )
 
     def change_object_cost(
@@ -824,7 +822,7 @@ class NewEffectSupport:
             stone: Union[int, None] = None,
             gold: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_OBJECT_COST,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -840,7 +838,7 @@ class NewEffectSupport:
             target_player: Union[int, None] = None,
             visibility_state: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.SET_PLAYER_VISIBILITY,
             source_player=source_player,
             target_player=target_player,
@@ -858,9 +856,9 @@ class NewEffectSupport:
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
             object_list_unit_id_2: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_OBJECT_ICON,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -886,9 +884,9 @@ class NewEffectSupport:
             object_group: Union[int, None] = None,
             object_type: Union[int, None] = None,
             object_list_unit_id_2: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.REPLACE_OBJECT,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -910,7 +908,7 @@ class NewEffectSupport:
             string_id: Union[int, None] = None,
             message: Union[str, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_OBJECT_DESCRIPTION,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -924,7 +922,7 @@ class NewEffectSupport:
             string_id: Union[int, None] = None,
             message: Union[str, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_PLAYER_NAME,
             source_player=source_player,
             string_id=string_id,
@@ -938,7 +936,7 @@ class NewEffectSupport:
             object_list_unit_id_2: Union[int, None] = None,
             button_location: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_TRAIN_LOCATION,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -953,7 +951,7 @@ class NewEffectSupport:
             object_list_unit_id_2: Union[int, None] = None,
             button_location: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_RESEARCH_LOCATION,
             source_player=source_player,
             technology=technology,
@@ -967,7 +965,7 @@ class NewEffectSupport:
             string_id: Union[int, None] = None,
             message: Union[str, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_CIVILIZATION_NAME,
             source_player=source_player,
             string_id=string_id,
@@ -983,9 +981,9 @@ class NewEffectSupport:
             area_x2: Union[int, None] = None,
             area_y2: Union[int, None] = None,
             object_list_unit_id_2: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CREATE_GARRISONED_OBJECT,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -1001,7 +999,7 @@ class NewEffectSupport:
             self,
             ai_signal_value: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.ACKNOWLEDGE_AI_SIGNAL,
             ai_signal_value=ai_signal_value,
         )
@@ -1013,24 +1011,27 @@ class NewEffectSupport:
             armour_attack_class: Union[int, None] = None,
             object_list_unit_id: Union[int, None] = None,
             source_player: Union[int, None] = None,
-            item_id: Union[int, None] = None,
             operation: Union[int, None] = None,
             object_attributes: Union[int, None] = None,
+            message: Union[str, None] = None,
     ) -> Effect:
         """
         The parameters 'armour_attack_quantity' and 'armour_attack_class' are only used when object_attributes is Armor
         or Attack (8 or 9). Use, 'quantity' otherwise.
         """
-        return self.trigger_ref._add_effect(
+        if (armour_attack_quantity is not None or armour_attack_class is not None) and quantity is not None:
+            raise ValueError("Cannot use 'armour_attack' attributes together with the 'quantity' attribute.")
+
+        return self._trigger_ref._add_effect(
             EffectId.MODIFY_ATTRIBUTE,
             quantity=quantity,
             armour_attack_quantity=armour_attack_quantity,
             armour_attack_class=armour_attack_class,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
-            item_id=item_id,
             operation=operation,
             object_attributes=object_attributes,
+            message=message
         )
 
     def modify_resource(
@@ -1038,15 +1039,13 @@ class NewEffectSupport:
             quantity: Union[int, None] = None,
             tribute_list: Union[int, None] = None,
             source_player: Union[int, None] = None,
-            item_id: Union[int, None] = None,
             operation: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.MODIFY_RESOURCE,
             quantity=quantity,
             tribute_list=tribute_list,
             source_player=source_player,
-            item_id=item_id,
             operation=operation,
         )
 
@@ -1054,15 +1053,13 @@ class NewEffectSupport:
             self,
             tribute_list: Union[int, None] = None,
             source_player: Union[int, None] = None,
-            item_id: Union[int, None] = None,
             operation: Union[int, None] = None,
             variable: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.MODIFY_RESOURCE_BY_VARIABLE,
             tribute_list=tribute_list,
             source_player=source_player,
-            item_id=item_id,
             operation=operation,
             variable=variable,
         )
@@ -1077,9 +1074,9 @@ class NewEffectSupport:
             area_y1: Union[int, None] = None,
             area_x2: Union[int, None] = None,
             area_y2: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.SET_BUILDING_GATHER_POINT,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -1097,7 +1094,7 @@ class NewEffectSupport:
             string_id: Union[int, None] = None,
             message: Union[str, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.SCRIPT_CALL,
             string_id=string_id,
             message=message,
@@ -1110,7 +1107,7 @@ class NewEffectSupport:
             variable: Union[int, None] = None,
             message: Union[str, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_VARIABLE,
             quantity=quantity,
             operation=operation,
@@ -1122,7 +1119,7 @@ class NewEffectSupport:
             self,
             timer: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CLEAR_TIMER,
             timer=timer,
         )
@@ -1135,9 +1132,10 @@ class NewEffectSupport:
             area_y1: Union[int, None] = None,
             area_x2: Union[int, None] = None,
             area_y2: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            player_color: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_OBJECT_PLAYER_COLOR,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -1145,6 +1143,7 @@ class NewEffectSupport:
             area_y1=area_y1,
             area_x2=area_x2,
             area_y2=area_y2,
+            player_color=player_color,
             selected_object_ids=selected_object_ids,
         )
 
@@ -1155,15 +1154,17 @@ class NewEffectSupport:
             area_y1: Union[int, None] = None,
             area_x2: Union[int, None] = None,
             area_y2: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            message: Union[str, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_OBJECT_CIVILIZATION_NAME,
             string_id=string_id,
             area_x1=area_x1,
             area_y1=area_y1,
             area_x2=area_x2,
             area_y2=area_y2,
+            message=message,
             selected_object_ids=selected_object_ids,
         )
 
@@ -1176,9 +1177,10 @@ class NewEffectSupport:
             area_y1: Union[int, None] = None,
             area_x2: Union[int, None] = None,
             area_y2: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            message: Union[str, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_OBJECT_PLAYER_NAME,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -1187,6 +1189,7 @@ class NewEffectSupport:
             area_y1=area_y1,
             area_x2=area_x2,
             area_y2=area_y2,
+            message=message,
             selected_object_ids=selected_object_ids,
         )
 
@@ -1198,9 +1201,9 @@ class NewEffectSupport:
             area_y1: Union[int, None] = None,
             area_x2: Union[int, None] = None,
             area_y2: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.DISABLE_UNIT_TARGETING,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -1219,9 +1222,9 @@ class NewEffectSupport:
             area_y1: Union[int, None] = None,
             area_x2: Union[int, None] = None,
             area_y2: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.ENABLE_UNIT_TARGETING,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -1241,7 +1244,7 @@ class NewEffectSupport:
             stone: Union[int, None] = None,
             gold: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_TECHNOLOGY_COST,
             source_player=source_player,
             technology=technology,
@@ -1257,7 +1260,7 @@ class NewEffectSupport:
             source_player: Union[int, None] = None,
             technology: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_TECHNOLOGY_RESEARCH_TIME,
             quantity=quantity,
             source_player=source_player,
@@ -1271,7 +1274,7 @@ class NewEffectSupport:
             string_id: Union[int, None] = None,
             message: Union[str, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_TECHNOLOGY_NAME,
             source_player=source_player,
             technology=technology,
@@ -1286,7 +1289,7 @@ class NewEffectSupport:
             string_id: Union[int, None] = None,
             message: Union[str, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_TECHNOLOGY_DESCRIPTION,
             source_player=source_player,
             technology=technology,
@@ -1299,7 +1302,7 @@ class NewEffectSupport:
             source_player: Union[int, None] = None,
             technology: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.ENABLE_TECHNOLOGY_STACKING,
             source_player=source_player,
             technology=technology,
@@ -1310,7 +1313,7 @@ class NewEffectSupport:
             source_player: Union[int, None] = None,
             technology: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.DISABLE_TECHNOLOGY_STACKING,
             source_player=source_player,
             technology=technology,
@@ -1320,7 +1323,7 @@ class NewEffectSupport:
             self,
             ai_signal_value: Union[int, None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.ACKNOWLEDGE_MULTIPLAYER_AI_SIGNAL,
             ai_signal_value=ai_signal_value,
         )
@@ -1333,9 +1336,9 @@ class NewEffectSupport:
             area_y1: Union[int, None] = None,
             area_x2: Union[int, None] = None,
             area_y2: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.DISABLE_OBJECT_SELECTION,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -1354,9 +1357,9 @@ class NewEffectSupport:
             area_y1: Union[int, None] = None,
             area_x2: Union[int, None] = None,
             area_y2: Union[int, None] = None,
-            selected_object_ids: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
     ) -> Effect:
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.ENABLE_OBJECT_SELECTION,
             object_list_unit_id=object_list_unit_id,
             source_player=source_player,
@@ -1371,8 +1374,54 @@ class NewEffectSupport:
             self,
             quantity: Union[int, None] = None,
             color_mood: Union[int, None] = None):
-        return self.trigger_ref._add_effect(
+        return self._trigger_ref._add_effect(
             EffectId.CHANGE_COLOR_MOOD,
             quantity=quantity,
             color_mood=color_mood
         )
+
+    def enable_object_deletion(
+            self,
+            object_list_unit_id: Union[int, None] = None,
+            source_player: Union[int, None] = None,
+            area_x1: Union[int, None] = None,
+            area_y1: Union[int, None] = None,
+            area_x2: Union[int, None] = None,
+            area_y2: Union[int, None] = None,
+            item_id: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
+    ):
+        return self._trigger_ref._add_effect(
+            EffectId.ENABLE_OBJECT_DELETION,
+            object_list_unit_id=object_list_unit_id,
+            source_player=source_player,
+            area_x1=area_x1,
+            area_y1=area_y1,
+            area_x2=area_x2,
+            area_y2=area_y2,
+            item_id=item_id,
+            selected_object_ids=selected_object_ids,
+        )
+
+    def disable_object_deletion(
+            self,
+            object_list_unit_id: Union[int, None] = None,
+            source_player: Union[int, None] = None,
+            area_x1: Union[int, None] = None,
+            area_y1: Union[int, None] = None,
+            area_x2: Union[int, None] = None,
+            area_y2: Union[int, None] = None,
+            item_id: Union[int, None] = None,
+            selected_object_ids: Union[int, List[int], None] = None,
+    ):
+        return self._trigger_ref._add_effect(
+            EffectId.DISABLE_OBJECT_DELETION,
+            object_list_unit_id=object_list_unit_id,
+            source_player=source_player,
+            area_x1=area_x1,
+            area_y1=area_y1,
+            area_x2=area_x2,
+            area_y2=area_y2,
+            item_id=item_id,
+            selected_object_ids=selected_object_ids,
+        )
