@@ -27,14 +27,13 @@ class Retriever:
         'log_value',
         '_data',
         'default_value',
-        # 'string_end_char'
     ]
 
     on_construct: RetrieverDependency
     on_commit: RetrieverDependency
     on_refresh: RetrieverDependency
 
-    def __init__(self, name, default_value, datatype=DataType(), is_list=None, log_value=False):
+    def __init__(self, name, default_value=None, datatype=DataType(), is_list=None, log_value=False):
         """
         Args:
             name (str): The name of the item. Has to be unique within the Section or Struct
@@ -55,8 +54,6 @@ class Retriever:
             self.datatype.log_value = True
             self.datatype._debug_retriever_name = name
 
-        # self.string_end_char = False
-
     def get_data_as_bytes(self):
         self.update_datatype_repeat()
 
@@ -100,6 +97,10 @@ class Retriever:
         if self.log_value:
             old_value = self._data
             self._print_value_update(old_value, value)
+
+        # If repeat is 0 and value being said is truthy (mainly not an empty list) set repeat to one
+        if self.datatype.repeat == 0 and value:
+            self.datatype.repeat = 1
         self._data = value
 
     def set_data_to_default(self):
@@ -169,9 +170,9 @@ class Retriever:
                 data = pretty_format_list(self.data)
             else:
                 data = string_manipulations.q_str(self.data)
-            return self.name + " (" + self.datatype.to_simple_string() + "): " + data
         else:
-            return "<None>"
+            data = "<None>"
+        return f"{self.name} ({self.datatype.to_simple_string()}): {data}"
 
     def to_simple_string(self):
         return f"[Retriever] {self.name}: {self.datatype} (Default: {string_manipulations.q_str(self.default_value)})"
